;+
; NAME:
; LISTDIR
;
; PURPOSE:
; Prints out a list of scans in a user-friendly way. For each scan in the list, the following information
; is shown: Scannum Nb_subscan Obs_Date UTC_Time Source Scan_Type PWV Project_ID
;
; CALLING SEQUENCE:
; listdir[, inlist=inlist]
; 
; INPUTS:
; None.
;
; OPTIONAL INPUT:
; inlist : list of indices in apexlog_struct to restrict  
; 
; OUTPUTS:
; None.
;
; COMMON BLOCKS:
; OBS1_CONFIGB: Contains working directory, calibration table and project name
;
; IMPORTANT NOTE:
; In order to work properly, the procedure build_apexobslog must have been ran already with this work_dir
;
; EXAMPLE:
; restore, work_dir+'apexdata/obslogs/apex_obslog.xdr', /verb
; w48 = where(strmid(apexlog_struct.source,0,3) eq 'W48')
; listdir, inlist=w48
; 
; MODIFICATION HISTORY:
;  - 20170126 - F. Schuller : created
;  - 20170911 - F. Schuller : added optional argument inlist
;-

pro listdir,inlist=inlist
  COMMON obs1_configb
  apexdata=work_dir+'apexdata/'
  restore, apexdata+'obslogs/apex_obslog.xdr', /verb  ; this assumes that build_apexobslog has already been ran with this work_dir

  if keyword_set(inlist) then files = apexlog_struct[inlist] $
     else files = apexlog_struct
  nb = n_elements(files)

  print,"Scan   Nsub Obs_Date   UTC_Time  Source       Type     PWV   Project"
  fmt = '(I-6," ",I-5,A10," ",A8,"  ",A-12," ",A-9,F4.2,"  ",A18)'
  for i=0,nb-1 do begin
    raw_files = findfile(apexdata+'rawdata/*'+strtrim(string(files[i].number),1)+'*', count=n_files)
    if n_files ge 1 then $
      print,string(files[i].number,files[i].numsubscans,files[i].date,files[i].UTC,files[i].source,$
                   files[i].type,files[i].pwv,files[i].projectid,format=fmt)
  endfor
end
