function clean_chaine, chaine, car1, car2

if n_params() ne 3 then begin
	print, 'Calling sequence :'
	print, '	chaine_propre = CLEAN_CHAINE(chaine, car1, car2)'
	return, -1
endif

pos = strpos(chaine, car1)
if pos ne -1 then strput, chaine, car2, pos

return, chaine
end

;===============================================================================

function header2string, commande, param_liste, header

if n_params() ne 3 then begin
	print, 'Calling sequence :'
	print, '	data_struct = HEADER2STRING(commande, param_liste, header)'
	return, -1
endif

chaine = commande

for i=0,n_elements(param_liste)-1 do begin
	value = fxpar(header, param_liste(i))
	
;	print, param_liste(i), value, size(value, /type)
	if size(value, /type) eq 7 then begin
;		value = clean_chaine(value, '/', ' ')
;		value = clean_chaine(value, '=', ' ')
;		value = clean_chaine(value, "'", " ")
;		value = clean_chaine(value, "'", " ")
		value = strtrim(value, 2)
		chaine = chaine + ", " + $
		           clean_chaine(param_liste(i), '-', '_') + ": '" + $
			   value + "'"
	endif else begin
		chaine = chaine + ', ' + $
		           clean_chaine(param_liste(i), '-', '_') + ': '  + $
			   string(value)
	endelse
endfor

return, chaine
end

;===============================================================================

;+
; NAME:
;	READFITS_APEX
;
; PURPOSE:
;	Put MBFits format into IDL structure file.	
;
; CALLING SEQUENCE:
;	Datastr = READFITS_APEX(Scan_number, Subscan, band=band, Calibration_camera=Calibration_camera)
;
; INPUTS:
;	Scan_number:	Scan number.
;	Subscan:	Subscan number.
;	Calibration_camera:	Calibration file.
;       Band:           Spectral band, 350, 450 or 200, default is 350
;	
; KEYWORD PARAMETERS:
;	NOPOWERMAP:	Do not Calibrate V to pW.
;		
; COMMON BLOCKS:
;	OBS1_CONFIGB:	Contains working directory, project and calibration table name.
;		
; EXAMPLE:
;		Datastr = READFITS_APEX_350(47527, 10, Calibration_camera=Calibration_camera, /NOPOWERMAP)
;
; MODIFICATION HISTORY:
;  - 2014     - P. Gallais  : created
;  - 2016     - P. Gallais  : 'band' argument added
;  - 20170912 - F. Schuller : adapted to raw data stored in a single FITS file
;  - 20170912 - F. Schuller : test if scandir is 'X' or 'Y' before computing scantime and scanlen
;  - 20171027 - Ph. Andr :   storing scanning direction information in datastr.obslog.map.direction even if telescope obslog is not available (NB: requires init_logstructure.pro)
;  - 20171108 - F. Schuller : removed cube_orig field from datastr
;-

function readfits_apex, scan_number, subscan, band=band, calibration_camera=calibration_camera, goodpix_ima=goodpix_ima, nopowermap=nopowermap 
COMMON obs1_configb

  if not keyword_set(band) then spectral_band=350 else spectral_band=band
  nopowermap = 1      ;;;;;;;;;;;; WARNING POWERMAP debranche

  apexdata = work_dir+'apexdata/'
  Calib_partemis = work_dir+'Calib_partemis/'
  path = apexdata+'rawdata/'

  if n_params() eq 0 then begin
	  print, 'Calling sequence :'
	  print, '	datastr = READFITS_APEX(scan_number, subscan, band=band, calibration_camera=calibration_camera, goodpix_ima=goodpix_ima, nopowermap=nopowermap)'
	  return, -1
  endif

  obsname = 'APEX-' + strtrim(string(scan_number),1) + '-*'
  subscan_num = strtrim(strcompress(subscan),2)
  base_name = path + obsname
  file_list  = file_search(base_name, count=nfiles)

  if file_test(file_list[0], /dir) then begin
    ; the raw data is stored in the MB-FITS direcory structure
    subscan_name = base_name + '/' + subscan_num
    liste_arraydata = findfile(subscan_name + '/ARTEMIS' + strtrim(spectral_band,2) + '*ARRAYDATA-1.fits', count=nbre_1)
    liste_datapar = findfile(subscan_name + '/ARTEMIS' + strtrim(spectral_band,2) + '*DATAPAR.fits', count=nbre_2)
    liste_headertel = findfile(base_name + '/SCAN.fits', count=nombre)

    if nbre_1 eq 0 or nbre_2 eq 0 then begin
       print, '% READFITS_APEX : Directory ' + path + obsname + ' not found'       
       openW, lun, work_dir + 'apexdata/' + strcompress('Error_report_on_' + string(scan_number)) + '_data.txt' , /APPEND, /GET_LUN  ; write into the error report
       printf, lun, strcompress(string(subscan)), '  : LES FICHIERS ARRAYDATA, DATAPAR ET SCAN N ONT PAS ETE TROUVES  :' + path + obsname
       close, lun
       free_lun, lun       
       return, -1
    endif
  endif else if file_test(file_list[0]) then begin
    ; single .fits file for this scan
    ; This file should follow the structure:
    ; index table
    ; 1     SCAN
    ; 2     FEBEPAR (ARTEMIS450)
    ; 3     FEBEPAR (ARTEMIS350)
    ; 4     DATAPAR (ARTEMIS450, subscan 1)
    ; 5     DATAPAR (ARTEMIS350, subscan 1)
    ; 6     ARRAYDATA (ARTEMIS450, subscan 1, HK data)
    ; 7     ARRAYDATA (ARTEMIS450, subscan 1, bolo data)
    ; 8     ARRAYDATA (ARTEMIS350, subscan 1, HK data)
    ; 9     ARRAYDATA (ARTEMIS350, subscan 1, bolo data)
    ; 10    MONITOR (subscan 1)
    ; 11    DATAPAR (ARTEMIS450, subscan 2)
    ; etc.
    liste_headertel = 1
    if spectral_band eq 350 then begin
      liste_arraydata = 7*fix(subscan_num) + 2
      liste_datapar   = 7*fix(subscan_num) - 2
    endif else begin   ; band = 450
      liste_arraydata = 7*fix(subscan_num) + 0
      liste_datapar   = 7*fix(subscan_num) - 3
    endelse
  endif else begin
    print,'No file found for scan '+strtrim(string(scan_number),1)
    return,-1
  endelse

  obslog = read_apexobslog(scan_number)	    	    	    	;  Structure if scan_number present in apex_obslog.xdr, -1 otherwise
  u = mrdfits(calibration_camera(0),1)

  ;;;;;;;;;;;;;;;;;;;;
  ; Read ARRAYDATA
  if datatype(liste_arraydata) eq 'INT' then begin
    arraydata = mrdfits(file_list[0], liste_arraydata, header_arraydata)
  endif else begin
    arraydata = mrdfits(liste_arraydata(0), 1, header_arraydata)
  endelse
  
  ; test that arraydata could be read
  if datatype(arraydata) ne 'STC' then begin
    print, '% READFITS_APEX : PROBLEM DETECTED IN ARRAYDATA AT SUBSCAN' + ' ' + subscan
    print, 'SUBSCAN' + ' ' + subscan + ' '+'IGNORED'
       
    openW, lun, work_dir + 'apexdata/' + strcompress('Error_report_on_' + string(scan_number)) + '_data.txt' , $
           /APPEND, /GET_LUN     ; write into the error report
    printf, lun, strcompress(string(subscan)), '  : ECHEC DE LA LECTURE DE ARRAYDATA AVEC MRDFITS, ARRAYDATA N EST PAS UNE STRUCTURE'
    close, lun
    free_lun, lun
    return, -1
  endif

  taille = size(arraydata.data)
  bidon2 = arraydata.data
  if (arraydata(0).mjd lt 56505.5) then bidon2 = -bidon2						;   Changing sign of data before 1 Aug. 2013 00h00 UTC

; 350 microns focal plane : 64 x 36 = 2304 pixels
n_channel_x = 64
n_channel_y = 36

cube = fltarr(64,36,taille(3))

flag_1150 = 0
if taille(2) eq 1150 then begin
	donnee = reform(bidon2, 1150, taille(3))
	bidon = fltarr(1152, taille(3))
;	bidon(0:1150-1,*) = donnee(*,*)
        bidon(0:575-1,*) = donnee(0:575-1,*)
        bidon(575:1150-1,*) = donnee(575:1150-1,*)	
	donnee = bidon
	flag_1150 = 1
endif


flag_1151 = 0
if taille(2) eq 1151 then begin
	donnee = reform(bidon2, 1151, taille(3))
	bidon = fltarr(1152, taille(3))
	bidon(0:1151-1,*) = donnee(*,*)
	donnee = bidon
	flag_1151 = 1
endif

if taille(2) eq 2304 then donnee = reform(bidon2, 2304, taille(3))
if taille(2) eq 1152 then donnee = reform(bidon2, 1152, taille(3))
if taille(2) eq 288 then donnee = reform(bidon2, 288, taille(3))
if taille(2) eq 2016 then donnee = reform(bidon2, 2016, taille(3))  	    ;  > April 2017 (PhA)
if taille(2) eq 864 then donnee = reform(bidon2, 864, taille(3))  	    ;  > April 2017 (PhA)

taille = size(donnee)

bidon=fltarr(18,18)


if taille(1) eq 288 then begin
	for i=0, taille(2)-1 do begin
		bidon(0:288-1)=donnee(0:288-1,i)
		bidon = rot(bidon,-90)
		array7 = bidon(2:*,*)

		plane = fltarr(64,36)
		plane(32:47,18:*) = array7

		cube(*,*,i) = plane		
	endfor
endif

if taille(1) eq 864 and spectral_band eq 450 then begin

	for i=0,taille(2)-1 do begin
		bidon(0:288-1)=donnee(0:288-1,i)
		bidon = rot(bidon, 90)
		array2 = bidon(0:15,*)	
	
		bidon(0:288-1)=donnee(288:288+288-1,i)
		bidon = rot(bidon, -90)
		array7 = bidon(0:15,*)

		bidon(0:288-1)=donnee(576:576+288-1,i)
		bidon = rot(bidon, -90)
		array8 = bidon(0:15,*)
		
		plane = fltarr(64,36)
;		plane(0:15,0:17) = array1
		plane(16:31,0:17) = array2
;		plane(32:47,0:17) = array3
;		plane(48:63,0:17) = array4
;		plane(0:15,18:*) = array5
;		plane(16:31,18:*) = array6
		plane(32:47,18:*) = array7
		plane(48:63,18:*) = array8

		cube(*,*,i) = plane
	endfor
endif

if taille(1) eq 2016 and spectral_band eq 350 then begin

;print, " Spectral band = 350"

	for i=0,taille(2)-1 do begin
		bidon(0:288-1)=donnee(0:288-1,i)
		bidon = rot(bidon, 90)
		array1 = bidon(0:15,*)	
	
		bidon(0:288-1)=donnee(288:288+288-1,i)
		bidon = rot(bidon, 90)
		array2 = bidon(0:15,*)

		bidon(0:288-1)=donnee(576:576+288-1,i)
		bidon = rot(bidon, 90)
		array3 = bidon(0:15,*)

		bidon(0:288-1)=donnee(864:864+288-1,i)
		bidon = rot(bidon, -90)
		array5 = bidon(0:15,*)		

		bidon(0:288-1)=donnee(1152:1152+288-1,i)
		bidon = rot(bidon,-90)
		array6 = bidon(2:*,*)

		bidon(0:288-1)=donnee(1440:1440+288-1,i)
		bidon = rot(bidon,-90)
		array7 = bidon(2:*,*)

		bidon(0:288-1)=donnee(1728:1728+288-1,i)
		bidon = rot(bidon,-90)
		array8 = bidon(2:*,*)
		
		plane = fltarr(64,36)
		plane(0:15,0:17) = array1
		plane(16:31,0:17) = array2
		plane(32:47,0:17) = array3
;;;		plane(48:63,0:17) = array4   	    	    	;  Dead sub-array at 350mu
		plane(0:15,18:*) = array5
		plane(16:31,18:*) = array6
		plane(32:47,18:*) = array7
		plane(48:63,18:*) = array8

		cube(*,*,i) = plane
	endfor
	
endif

if taille(1) eq 1152 then begin
	for i=0, taille(2)-1 do begin
		bidon(0:288-1)=donnee(0:288-1,i)
		bidon = rot(bidon, 90)
		array2 = bidon(0:15,*)
		
		bidon(0:288-1)=donnee(288:288+288-1,i)
		bidon = rot(bidon, 90)
		array3 = bidon(0:15,*)

		bidon(0:288-1)=donnee(576:576+288-1,i)
		bidon = rot(bidon,-90)
		array6 = bidon(2:*,*)

		bidon(0:288-1)=donnee(864:864+288-1,i)
		bidon = rot(bidon,-90)
		array7 = bidon(2:*,*)

		plane = fltarr(64,36)
		plane(16:31,0:17) = array2
		plane(32:47,0:17) = array3
		plane(16:31,18:*) = array6
		plane(32:47,18:*) = array7

		cube(*,*,i) = plane
		
	endfor

endif else if taille(1) eq 2304 then begin

	for i=0,taille(2)-1 do begin
		bidon(0:288-1)=donnee(0:288-1,i)
		bidon = rot(bidon, 90)
		array1 = bidon(0:15,*)	
	
		bidon(0:288-1)=donnee(288:288+288-1,i)
		bidon = rot(bidon, 90)
		array2 = bidon(0:15,*)

		bidon(0:288-1)=donnee(576:576+288-1,i)
		bidon = rot(bidon, 90)
		array3 = bidon(0:15,*)

		bidon(0:288-1)=donnee(864:864+288-1,i)
		bidon = rot(bidon, 90)
		array4 = bidon(0:15,*)		

		bidon(0:288-1)=donnee(1152:1152+288-1,i)
		bidon = rot(bidon,-90)
		array5 = bidon(2:*,*)

		bidon(0:288-1)=donnee(1440:1440+288-1,i)
		bidon = rot(bidon,-90)
		array6 = bidon(2:*,*)

		bidon(0:288-1)=donnee(1728:1728+288-1,i)
		bidon = rot(bidon,-90)
		array7 = bidon(2:*,*)

		bidon(0:288-1)=donnee(2016:2016+288-1,i)
		bidon = rot(bidon,-90)
		array8 = bidon(2:*,*)
		
		plane = fltarr(64,36)
		plane(0:15,0:17) = array1
		plane(16:31,0:17) = array2
		plane(32:47,0:17) = array3
		plane(48:63,0:17) = array4
		plane(0:15,18:*) = array5
		plane(16:31,18:*) = array6
		plane(32:47,18:*) = array7
		plane(48:63,18:*) = array8

		cube(*,*,i) = plane

	endfor

endif

;cube = reform(bidon2, n_channel_x, n_channel_y, taille(3))
if spectral_band eq 450 then cube = img2focal450(cube) else cube = img2focal350(cube)
;cube = img2focal350(cube)       ; Cube avec les inter-matrices



;;;;;;;;;;;;;;;;;;;;
; Read DATAPAR
  if datatype(liste_datapar) eq 'INT' then begin
    datapar = mrdfits(file_list[0], liste_datapar, header_datapar)
  endif else begin
    datapar = mrdfits(liste_datapar(0), 1, header_datapar)
  endelse

  if datatype(datapar) ne 'STC' then begin
    print, '% READFITS_APEX : PROBLEM DETECTED IN DATAPAR AT SUBSCAN' + ' ' + subscan
    print, 'SUBSCAN' + ' ' + subscan + ' '+'IGNORED'
       
    openW, lun, work_dir + 'apexdata/' + strcompress('Error_report_on_' + string(scan_number)) + '_data.txt' , $
           /APPEND, /GET_LUN ; write into the error report
    printf, lun, strcompress(string(subscan)), '  : ECHEC DE LA LECTURE DE DATAPAR AVEC MRDFITS, DATAPAR N EST PAS UNE STRUCTURE'
    close, lun
    free_lun, lun
    return, -1
  endif

;hkcube = readfits(liste(2), header_chop)
;poschop = fltarr(n_elements(hkcube(2,*)))
poschop = intarr(n_elements(cube(0,0,*)))
;poschop(*) = hkcube(2,*)*20/65535-10
chopbeam = fix(poschop*0)
;lst = indgen(n_elements(hkcube(2,*)), /double)
lst = indgen(n_elements(cube(0,0,*)), /double)

;;;;;;;;;;;;;;;;;;;;
; Read SCAN table
  if datatype(liste_headertel) eq 'INT' then begin
    datatel = mrdfits(file_list[0], liste_headertel, header_tel)
  endif else begin
    datatel = mrdfits(liste_headertel(0), 1, header_tel)
  endelse

; Modification du header pour integrer les parametres provenant de APEX avec la
; bonne denomination dans la structure

id = where(strmid(header_tel, 0, 4)  eq  'LST ')
bidon = header_tel(id(0))
strput, bidon, 'LST_INI', 0
header_tel(id(0)) = bidon

fxaddpar, header_tel, 'CHOPAMP', fxpar(header_tel, 'WOBTHROW')				;  Amplitude (1/2 throw) of the wobbler in deg
fxaddpar, header_tel, 'LAMBDA',  fxpar(header_tel, 'BLONGOBJ')
fxaddpar, header_tel, 'BETA',    fxpar(header_tel, 'BLATOBJ')
fxaddpar, header_tel, 'SOURCE',  fxpar(header_tel, 'OBJECT')
fxaddpar, header_tel, 'OTFVLAM', fxpar(header_tel, 'SCANXVEL')
fxaddpar, header_tel, 'OBSMODE', fxpar(header_tel, 'SCANTYPE')
fxaddpar, header_tel, 'NIMGCHOP',  fxpar(header_tel, 'WOBCYCLE') ;  Attention, le contenu est recalcule plus tard
fxaddpar, header_tel, 'SUBSCAN',  subscan ;  Attention, le contenu est recalcule plus tard
fxaddpar, header_tel, 'SPECTIME',  fxpar(header_tel, 'SCANTIME')

obs_cmd = fxpar(header_tel, 'OBSCMD')
; read calibration_camera FITS file - only need the header
toto = mrdfits(calibration_camera(0),1, header_cam)

fxaddpar, header_tel, 'VL_0',  fxpar(header_cam, 'VL_0')
fxaddpar, header_tel, 'VH_0',  fxpar(header_cam, 'VH_0')
fxaddpar, header_tel, 'VH_BLIND_0',  fxpar(header_cam, 'VH_BLIND_0')
fxaddpar, header_tel, 'VL_1',  fxpar(header_cam, 'VL_1')
fxaddpar, header_tel, 'VH_1',  fxpar(header_cam, 'VH_1')
fxaddpar, header_tel, 'VH_BLIND_1',  fxpar(header_cam, 'VH_BLIND_1')
fxaddpar, header_tel, 'VL_2',  fxpar(header_cam, 'VL_2')
fxaddpar, header_tel, 'VH_2',  fxpar(header_cam, 'VH_2')
fxaddpar, header_tel, 'VH_BLIND_2',  fxpar(header_cam, 'VH_BLIND_2')
fxaddpar, header_tel, 'VL_3',  fxpar(header_cam, 'VL_3')
fxaddpar, header_tel, 'VH_3',  fxpar(header_cam, 'VH_3')
fxaddpar, header_tel, 'VH_BLIND_3',  fxpar(header_cam, 'VH_BLIND_3')

; parametres_cam_1 = [ $   ; voir readfits_apex version 2009 pour liste parametres
; parametres_cam_2 = [ $   ; voir readfits_apex version 2009 pour liste parametres

parametres_tel_1 = [ $   
'EXTNAME ', $;  'SCAN-MBFITS'	  / name of this binary table extension
'TELESCOP', $;  'APEX-12m' 	  / Telescope Name
'SITELONG', $; 	-67.7592222222222 / [deg] Observatory longitude
'SITELAT ', $; 		-23.00575 / [deg] Observatory latitude
'SITEELEV', $; 		    5105. / [m] Observatory elevation
'DIAMETER', $; 		      12. / [m] Dish diameter
'PROJID  ', $;  'T-78.F-0006-2006'   / Project ID
'OBSID   ', $;  'RS/NN   ' 	  / Observer and operator initials
'SCANNUM ', $; 		     4025 / Scan number
'SUBSCAN ', $;                  9 / Subscan number (not part of the original header)
'TIMESYS ', $;  'TAI     ' 	  / Time system for MJD and DATE-OBS
'DATE-OBS', $;  '2007-03-20T15:01:03' / Scan start in TIMESYS system (Y2K format with
'MJD	 ', $; 	 54179.6257291667 / Scan date/time (Modified Julian Date) in
'LST_INI ', $; 	  80432.059377301 / Local apparent sidereal time (scan start)
'NOBS	 ', $; 			1 / Number of subscans in this scan
'NSUBS   ', $; 			1 / Number of subscans in this scan
'UTC2UT1 ', $; 		 -0.05192 / [s] UT1-UTC time translation
'TAI2UTC ', $; 		      33. / [s] UTC-TAI time translation
'ETUTC   ', $; 		    -999. / [s] Ephemeris Time - UTC time translation
'GPSTAI  ', $; 		      19. / [s] GPS time - TAI translation
'CTYPE1  ', $;  'RA---SFL' 	  / Basis system (longitude)
'CTYPE2  ', $;  'DEC--SFL' 	  / Basis system (latitude)
'WCSNAME ', $;  'absolute equatorial' / Human-readable description of frame
'RADESYS ', $;  'FK5     ' 	  / Additional system definition for ecliptic/equat
'EQUINOX ', $; 		    2000. / [Julian years] Equinox
'CRVAL1  ', $; 	 258.380402937614d0 / [deg] Native frame zero in basis system (long.)
'CRVAL2  ', $; 	 -22.283130251835d0 / [deg] Native frame zero in basis system (lat.)
'LONPOLE ', $; 		     180. / [deg] Native longitude of celestial pole: range
'LATPOLE ', $; 	  67.716869748165 / [deg] Basis latitude of native pole
'OBJECT  ', $;  'Jupiter ' 	  / Source name
'BLONGOBJ', $; 	  258.38040090495d0 / [deg] Source longitude in basis frame
'BLATOBJ ', $; 	-22.2831301671406d0 / [deg] Source latitude in basis frame
'LONGOBJ ', $;  -2.03266398557389E-06 / Source longitude in user native frame
'LATOBJ  ', $;  8.46943990495674E-08 / Source latitude in user native frame
'CALCODE ', $;  '        ' 	  / Calibrator Code
'MOVEFRAM', $; 			F / True if tracking a moving frame
'PERIDATE', $; 		    -999. / [Julian days] TP, Full Julian date of perihelio
'PERIDIST', $; 		    -999. / [AU] QR, perihelion distance
'LONGASC ', $; 		    -999. / [deg] OM, Longitude of ascending node
'OMEGA   ', $; 		    -999. / [deg] W, Angle from asc. node to perihelion
'INCLINAT', $; 		    -999. / [deg] IN, Inclination
'ECCENTR ', $; 		    -999. / EC, Eccentricity
'ORBEPOCH', $; 		    -999. / [Julian days] EPOCH, Epoch of orbital elements
'ORBEQNOX', $; 		    2000. / [years] Elements equinox: J2000.0 or B1950.0
'DISTANCE', $; 		    -999. / [AU] Geocentric Distance
'SCANTYPE', $;  'ONOFF   ' 	  / Scan astronomical type
'SCANMODE', $;  'RASTER  ' 	  / Mapping mode
'SCANGEOM', $;  'SINGLE  ' 	  / Scan geometry
'SCANDIR ', $;  'Unkn    ' 	  / (optional) scan direction
'SCANLINE', $; 			0 / (optional) number of lines in a scan
'SCANRPTS', $; 			0 / (optional) number of repeats of each scan line
'SCANLEN ', $; 		      -0. / (optional, OTF/RASTER) line length in deg
'SCANXVEL', $; 		      -0. / (optional, OTF) tracking rate along line.
'SCANTIME', $; 		      -0. / (optional, OTF) time for one line
'SCANXSPC', $; 		      -0. / (optional, RASTER) step along line between samp
'SCANYSPC', $; 		      -0. / (optional, OTF/RASTER) step between scan/raster
'SCANSKEW', $; 		      -0. / (optional, OTF/RASTER) offset in scan direction
'SCANPAR1', $; 		      -0. / (optional) spare scan parameter
'SCANPAR2', $; 		      -0. / (optional) another spare scan parameter
'CROCYCLE', $;  '        ' 	  / CAL/REF/ON loop string
'ZIGZAG  ', $; 			F / (optional, OTF/RASTER) Scan in zigzag?
'TRANDIST', $; 		    -999. / [m] (optional, HOLO) Holography transmitter dis
'TRANFREQ', $; 		    -999. / [Hz] (optional, HOLO) Holography transmitter fr
'TRANFOCU', $; 		    -999. / [deg] (optional, HOLO) Holography transmitter o
'WOBUSED ', $; 			F / Wobbler used?
'WOBTHROW', $; 		       0. / [deg] Wobbler throw
'WOBDIR  ', $;  'NONE    ' 	  / Wobbler throw direction
'WOBCYCLE', $; 		       0. / [s] Wobbler period
'WOBMODE ', $;  'NONE    ' 	  / Wobbler mode (SQUARE/TRIANGULAR)
'WOBPATT ', $;  'NONE    ' 	  / Wobbler pattern (e.g. NEG,POS,SYM)
'PHASE1  ', $;  'NONE    ' 	  / Phase 1 description
'NFEBE   ', $; 			1 / Number of FEBEs
'PDELTACA', $; 		       0. / [deg] Accumulated user ptg correction CA
'PDELTAIE', $; 		       0. / [deg] Accumulated user ptg correction IE
'FDELTACA', $; 		       0. / [deg] Accumulated ptg corr to CA due to focus
'FDELTAIE', $; 		       0. / [deg] Accumulated ptg corr to IE due to focus
'IA	 ', $; 		 0.418497 / [deg] Pointing Coefficient (-P1)
'IE	 ', $; 		0.4790366 / [deg] Pointing Coefficient (P7)
'HASA	 ', $; 	     -0.001058417 / [deg] Pointing Coefficient
'HACA	 ', $; 	    -0.0002590278 / [deg] Pointing Coefficient
'HESE	 ', $; 		       0. / [deg] Pointing Coefficient (was ZFLX)
'HECE	 ', $; 	     -0.006228222 / [deg] Pointing Coefficient (P8; was ECEC)
'HESA	 ', $; 	     2.183333E-05 / [deg] Pointing Coefficient
'HASA2   ', $; 	    -0.0008196944 / [deg] Pointing Coefficient
'HACA2   ', $; 	     -0.001612556 / [deg] Pointing Coefficient
'HESA2   ', $; 	    -5.102778E-05 / [deg] Pointing Coefficient
'HECA2   ', $; 	      -0.00082525 / [deg] Pointing Coefficient
'HACA3   ', $; 	     0.0005024445 / [deg] Pointing Coefficient
'HECA3   ', $; 	     0.0002202222 / [deg] Pointing Coefficient
'HESA3   ', $; 	     1.136111E-05 / [deg] Pointing Coefficient
'NPAE	 ', $; 		   -0.005 / [deg] Pointing Coefficient (-P3)
'CA	 ', $; 	      -0.03923142 / [deg] Pointing Coefficient (-P2)
'AN	 ', $; 	      0.006855028 / [deg] Pointing Coefficient (-P5)
'AW	 ', $; 	     -0.008293056 / [deg] Pointing Coefficient (-P4)
'TTYPE1  ', $;  'FEBE    ' 	  / Frontend-backend combination identification
'TFORM1  ', $;  '17A     ' 	  / format of field
'CHECKSUM', $;  'i6aAj6T4i6Y9i6Y9'   / HDU checksum updated 2007-03-20T15:01:11
'DATASUM ', $;  '1595939336'	  / data unit checksum updated 2007-03-20T15:01:11
; Ici,on trouve les parametres pour le traitement similaire Kosma
'CHOPAMP ', $;                     / equiv 'WOBTHROW'
'LAMBDA  ', $;                     / equiv 'BLONGOBJ'
'BETA    ', $;  		   / equiv 'BLATOBJ'
'SOURCE  ', $;                     / equiv 'OBJECT'
'OTFVLAM ', $;                     / equiv 'SCANXVEL'
'SPECTIME', $;                     / equiv 'SCANTIME'
'OTFSBET' , $; 
'NIMGCHOP', $;                     / equiv 'WOBCYCLE' a quelque chose pres
'OBSMODE',  $;                     / equiv 'SCANTYPE'
;;;;;'OFFSET',   $;			L'offset detecteur provenant des HK
'VL_0',         $;         		V low  detecteur
'VH_0',         $;			V high detecteur
'VH_BLIND_0',   $;			V compensation mode differentiel
'VL_1',         $;         		V low  detecteur
'VH_1',         $;			V high detecteur
'VH_BLIND_1',   $;			V compensation mode differentiel
'VL_2',         $;         		V low  detecteur
'VH_2',         $;			V high detecteur
'VH_BLIND_2',   $;			V compensation mode differentiel
'VL_3',         $;         		V low  detecteur
'VH_3',         $;			V high detecteur
'VH_BLIND_3'    $;			V compensation mode differentiel
]

parametres_tel_2 = [ $
'NUMSPEC ', $ 
'NAMESPEC'  ]


taille = size(cube)

; Construction de la structure house keeping
;commande = header2string('', parametres_cam_2, header_cam)
;commande = strmid(commande, 1, strlen(commande))
;commande = strcompress('hk = {' + commande + ', hkcube:hkcube}')
;status = execute(commande)

; Construction de la structure contenant des parametres telescopes secondaires
commande = header2string('', parametres_tel_2, header_tel)
commande = strmid(commande, 1, strlen(commande))
commande = strcompress('tel = {' + commande + '}')
status = execute(commande)

;subscan_num = strtrim(strcompress(subscan),2)

apexdata = work_dir+'apexdata/'	
dirdat = apexdata + 'rawdata/'              
u=find_all_dir(dirdat)
u=u[where(strmatch(u,'*'+strtrim(string(scan_number),1)+'*',/FOLD_CASE) EQ 1)] 
proj=u[0]
proj=strsplit(proj,/EXTRACT,'-')
taille2=size(proj)
taille2=taille2(1)
proj_name=proj(taille2-4:taille2-1)
proj_name=strjoin(proj_name,'-')

filename = 'APEX-' + strtrim(string(scan_number),1) + '_' + subscan_num + '-' + proj_name

; construction de la structure de donnees
commande = 'filename : filename, cube : cube'
commande = commande + ", band : " + strtrim(spectral_band,2)
commande = commande + ", azim : dblarr(" + strtrim(string(taille(3)), 2) + ")"
commande = commande + ", elev : dblarr(" + strtrim(string(taille(3)), 2) + ")"
commande = commande + ", rota : dblarr(" + strtrim(string(taille(3)), 2) + ")"
commande = commande + ', poschop : poschop'
commande = commande + ', chopbeam : chopbeam'
commande = commande + ', lst : lst'
commande = commande + ', lst_mid : double(0)'
commande = commande + ', alpha_mid : double(0)'
commande = commande + ', delta_mid : double(0)'
commande = commande + ', cdelt1 : 0.'
commande = commande + ', cdelt2 : 0.'
commande = commande + ', crota1 : 0.'
commande = commande + ', art_rot_angle : 0.'
commande = commande + ', art_xref : 0.'
commande = commande + ', art_yref : 0.'
commande = commande + ', daz_point : 0.'
commande = commande + ', del_point : 0.'
commande = commande + ', dra_point : 0.'
commande = commande + ', ddec_point : 0.'
commande = commande + ', pas_az : 0.'
commande = commande + ', unit   : "V"'
commande = commande + ', datapar: datapar'
commande = commande + ', elevatio : double(0)'
commande = commande + ', tau : 0.'
commande = commande + ', nefd_med : 0.'
commande = commande + ', scan : double(0)'
commande = commande + ', subscan_number : double(0)'

if datatype(obslog) eq 'STC' then begin
 commande = commande + ', obslog : obslog'
endif else begin
 init_logstructure, log_str_vide
 commande = commande + ', obslog : log_str_vide'
endelse 

commande = header2string(commande, parametres_tel_1, header_tel)

; 350 microns focal plane : 64 x 36 = 2304 pixels
; n_channel_x = 35 = taille(1)
; n_channel_y = 39 = taille(2)

taille =size(cube)                    ;;;;;; WARNING6436 etendu
nchannels=taille(1)*taille(2)

commande = commande + ', nchannels : ' + strtrim(nchannels)
commande = commande + ', freq : 40.'

commande = 'datastr = { ' + commande + $
            ',rmsmoy:fltarr(taille(1),taille(2)), goodpix_ima:bytarr(taille(1),taille(2))}'
;            ',rmsmoy:fltarr(taille(1),taille(2)), goodpix_ima:bytarr(taille(1),taille(2)), '+ $
;	   'cube_orig:cube}'
commande = strcompress(commande)

sub_com = strmid(commande,strpos(commande,'BLONGOBJ:'),50)
sub_com_ext = strmid(sub_com,0,strpos(sub_com,','))+'d0'+strmid(sub_com,strpos(sub_com,','),strlen(sub_com)-strpos(sub_com,','))
commande_ext1 = strmid(commande,0,strpos(commande,'BLONGOBJ:')+strpos(sub_com,','))+'d0'
commande_ext2 = strmid(commande,strpos(commande,'BLONGOBJ:')+strpos(sub_com,','),strlen(commande)-strpos(commande,'BLONGOBJ:')-strpos(sub_com,','))
commande = commande_ext1+commande_ext2
;
sub_com = strmid(commande,strpos(commande,'BLATOBJ :'),50)
sub_com_ext = strmid(sub_com,0,strpos(sub_com,','))+'d0'+strmid(sub_com,strpos(sub_com,','),strlen(sub_com)-strpos(sub_com,','))
commande_ext1 = strmid(commande,0,strpos(commande,'BLATOBJ :')+strpos(sub_com,','))+'d0'
commande_ext2 = strmid(commande,strpos(commande,'BLATOBJ :')+strpos(sub_com,','),strlen(commande)-strpos(commande,'BLATOBJ �:')-strpos(sub_com,','))
commande = commande_ext1+commande_ext2

;stop
status = execute(commande(0))

datastr.elevatio = median(datastr.datapar.elevatio)
datastr.lst = datastr.datapar.lst
; datastr.freq =  40.		;  Frequence d'acquisition en Hz

taille = size(datastr.cube)
datastr.nchannels = taille(1)*taille(2)

datastr.cdelt1 = 3.9  ; taille du pixel Artemis 350 microns a APEX en arcsec
datastr.cdelt2 = 3.9  ;

;bidon = extract_ima_moy(datastr.cube, datastr.rmsmoy)
bidon = datastr.chopamp*3600.							; Expected amplitude (1/2 throw) of the wobbler in arcsec

datastr.nimgchop = (datastr.wobcycle/2.) * datastr.freq				; Nb d'images par 1/2 cycle du wobbler

if datastr.wobused eq 1 and datastr.nimgchop gt 0 then begin
		datastr.chopbeam = raw2chopbeam_apex(datastr.datapar.wobdisln*3600., bidon)
;		print, 'READFITS_APEX : WARNING - Wobbler position to be determined from MBFITS words'
endif

;datastr.chopamp = bidon

if datastr.lambda eq 0 and datastr.beta eq 0 then begin
	altaz2hadec, datastr.elevatio, datastr.azimuth, datastr.sitelat, ha, dec
	alpha = lst_ini*15./3600. - ha
	if alpha lt 0 then alpha = 360 + alpha
	datastr.lambda = alpha
	datastr.beta = dec
endif

if flag_1151 eq 1 then goodpix_ima(19,38) = 0

datastr.goodpix_ima = goodpix_ima			;  Utilise dans do_map_scan pour la reconstruction du subscan OTF 

if datastr.scanmode eq 'OTF' then begin
    if datatype(obslog) eq 'STC' then begin
      ; check if scandir is 'X' or 'Y' - FSc 20171004
      if strupcase(datastr.obslog.map.direction) eq 'X' then begin
        datastr.scanxvel = float(datastr.obslog.map.xstep)/float(datastr.obslog.map.time) ;; in arcsec/sec
        datastr.otfvlam = double(datastr.scanxvel)
        datastr.scantime = float(datastr.obslog.map.xlen)/datastr.scanxvel  ; in sec
        datastr.scanlen  = float(datastr.obslog.map.xlen)/3600.              ; line length in degree
      endif else begin
        datastr.scanxvel = float(datastr.obslog.map.ystep)/float(datastr.obslog.map.time) ;; in arcsec/sec
        datastr.otfvlam = double(datastr.scanxvel)
        datastr.scantime = float(datastr.obslog.map.ylen)/datastr.scanxvel  ; in sec
        datastr.scanlen  = float(datastr.obslog.map.ylen)/3600.              ; line length in degree          
      endelse
    endif else begin
      print, "obslog = -1"
      pos_step0 = strpos(obs_cmd,'xstep=')
      pos_step1 = strpos(obs_cmd,',ylen=')
      xstep_cmd = strmid(obs_cmd,pos_step0+6,pos_step1-pos_step0-6)
      pos_ystep0 = strpos(obs_cmd,'ystep=')
      pos_ystep1 = strpos(obs_cmd,',time=')
      ystep_cmd = strmid(obs_cmd,pos_ystep0+6,pos_ystep1-pos_ystep0-6)    
      pos_time0 = strpos(obs_cmd,'time=')
      pos_time1 = strpos(obs_cmd,',direction=')
      time_cmd = strmid(obs_cmd,pos_time0+5,pos_time1-pos_time0-5)
      pos_len0 = strpos(obs_cmd,'xlen=')
      pos_len1 = strpos(obs_cmd,',xstep=')
      xlen_cmd = strmid(obs_cmd,pos_len0+5,pos_len1-pos_len0-5)
      pos_dir0 = strpos(obs_cmd,'direction=')
      pos_dir1 = strpos(obs_cmd,',zigzag=')
      dir_cmd  = strmid(obs_cmd,pos_dir0+11,pos_dir1-pos_dir0-12)
      datastr.obslog.map.direction = dir_cmd	    	    	    	    	    ; scanning direction read in from headertel of rawdata
      datastr.obslog.command = obs_cmd
;
      datastr.scanxvel = float(xstep_cmd)/float(time_cmd)   ;; in arcsec/sec
      datastr.otfvlam = double(datastr.scanxvel)	
      datastr.scantime = float(xlen_cmd)/datastr.scanxvel   ; in sec
      datastr.scanlen  = float(xlen_cmd)/3600.              ; line length in degree
    endelse	
  endif
  datastr.pas_az = double(datastr.otfvlam/datastr.freq)  		   ; pas le long de la direction de scanning en arcsec
  dimens=size(datastr.cube)
  datastr.pas_az = sgn(median(deriv(datastr.datapar.longoff)))*datastr.pas_az     ;  OK seulement si 0 le angle(scanning) le 90

  if not keyword_set(nopowermap) then begin
    power_map, datastr.vhblind, datastr.vh-datastr.vl, $			; 2009
	             datastr.vl, datastr.cube, cube_out
    datastr.cube = cube_out
    datastr.unit = 'pW'
  endif

datastr.movefram = 0
source = datastr.object
if (source eq 'Mercury' or source eq 'Venus' or source eq 'Mars' or source eq 'Jupiter' or source eq 'Saturn' or source eq 'Uranus' or source eq 'Neptune') then begin
	datastr.movefram = 1
	print, "Observing Planet ", source
endif

return, datastr

end
