pro slice_otf, datastr_in, dirdef, proj_name, keep=keep

; MODIFICATION HISTORY:
;  - 20171108 - F. Schuller - removed reference and use of cube_orig field in datastr
;-


; keep = 1 : on elimine les transitions entre scans
; keep = 2 : on garde toutes les donnees, meme les transitions entre scans 

command = datastr_in.obslog.command				;  recuperation de datastr_in additionnelles dans les OBSLOG
if strpos(command,'angle=') ne -1L then begin			; 'angle=' is contained in command
  command = strmid(command,strpos(command,'angle='))
endif else begin
  command = -1
endelse  
if ((size(command))(1) eq 7) then begin 			;  command is a string (i.e. strmid did not return -1)
	angle = (strsplit(command,',',/extract))(0)
	angle =  float(strmid(angle,strpos(angle,'=')+1))	; Rotation angle (deg) of scanning direction relative to Az axis
endif else begin
	angle = 0.0
endelse

angle = angle*!pi/180d0

scan_off = cos(angle)*datastr_in.datapar.longoff + sin(angle)*datastr_in.datapar.latoff
cross_scan_off = -sin(angle)*datastr_in.datapar.longoff + cos(angle)*datastr_in.datapar.latoff


;------------------ on entre dans le vif du sujet ------------------------------------------

;ind = indgen(n_elements(datastr_in.datapar.lst))
ind = lindgen(n_elements(datastr_in.datapar.lst))

scan_vel = deriv(datastr_in.datapar.lst, scan_off*3600)     	    	    ; Instantaneous scanning velocity in arcsec/sec

med_scan_vel = median(abs(scan_vel))	    	    	    	    	    ; Median absolute scanning velocity in arcsec/sec

ind_pos_vel = where(scan_vel gt med_scan_vel/4.)    	    	    	    ; Images with positive scanning velocities

ind_neg_vel = where(scan_vel lt -med_scan_vel/4.)   	    	    	    ; Images with negative scanning velocities

;;

datapar = datastr_in.datapar

npos = n_elements(ind_pos_vel)

dlst_pos = datapar(ind_pos_vel(1:npos-1)).lst-datapar(ind_pos_vel(0:npos-2)).lst

ind_end_pos = where(dlst_pos gt 100.*25e-3, cnt_end_pos)

ind_beg_pos = where(dlst_pos gt 100.*25e-3, cnt_end_pos)+1L

iend_pos_last = npos-1

ibeg_pos_first = 0

ind_end_pos = [where(dlst_pos gt 100.*25e-3, cnt_end_pos),iend_pos_last]

iend_pos_last = ind_end_pos(cnt_end_pos)

ind_beg_pos = [ibeg_pos_first,where(dlst_pos gt 100.*25e-3, cnt_end_pos)+1L]

ibeg_pos_first = ind_beg_pos(0)

med_pos_length = median(ind_end_pos-ind_beg_pos)

ind_keep_pos = where((ind_end_pos-ind_beg_pos) gt med_pos_length/10.,cnt_keep_pos)

if cnt_keep_pos lt n_elements(ind_end_pos) then begin
  ind_end_pos = ind_end_pos(ind_keep_pos)
  ind_beg_pos = ind_beg_pos(ind_keep_pos)
endif  

;;

nneg = n_elements(ind_neg_vel)

dlst_neg = datapar(ind_neg_vel(1:nneg-1)).lst-datapar(ind_neg_vel(0:nneg-2)).lst

ind_end_neg = where(dlst_neg gt 100.*25e-3, cnt_end_neg)

ind_beg_neg = where(dlst_neg gt 100.*25e-3, cnt_end_neg)+1L

iend_neg_last = nneg-1

ibeg_neg_first = 0

ind_end_neg = [where(dlst_neg gt 100.*25e-3, cnt_end_neg),iend_neg_last]

iend_neg_last = ind_end_neg(cnt_end_neg)

ind_beg_neg = [ibeg_neg_first,where(dlst_neg gt 100.*25e-3, cnt_end_neg)+1L]

ibeg_neg_first = ind_beg_neg(0)

med_neg_length = median(ind_end_neg-ind_beg_neg)

ind_keep_neg = where((ind_end_neg-ind_beg_neg) gt med_neg_length/10.,cnt_keep_neg)

if cnt_keep_neg lt n_elements(ind_end_neg) then begin
  ind_end_neg = ind_end_neg(ind_keep_neg)
  ind_beg_neg = ind_beg_neg(ind_keep_neg)
endif  

;

if n_elements(ind_end_pos) ne n_elements(ind_beg_pos) or n_elements(ind_end_neg) ne n_elements(ind_beg_neg) then begin
	print, ' Problem encountered slicing the data into subscans'
	print, 'ABORTING...'
	return	
        stop
endif

nscans_pos = n_elements(ind_end_pos)
nscans_neg = n_elements(ind_end_neg)

nscans = nscans_pos+nscans_neg

debut = lonarr(nscans)
fin = lonarr(nscans)

;if ind_pos_vel(0) lt ind_neg_vel(0) then begin
if ind_pos_vel(ind_beg_pos(0)) lt ind_neg_vel(ind_beg_neg(0)) then begin
 for kk=0, nscans_pos-1 do begin
   debut(2*kk) = ind_pos_vel(ind_beg_pos(kk))
   fin(2*kk) = ind_pos_vel(ind_end_pos(kk))
 endfor
 ;
 for kk=0, nscans_neg-1 do begin
   debut(2*kk+1) = ind_neg_vel(ind_beg_neg(kk))
   fin(2*kk+1) = ind_neg_vel(ind_end_neg(kk))
 endfor
endif

;

;if ind_pos_vel(0) gt ind_neg_vel(0) then begin
if ind_pos_vel(ind_beg_pos(0)) gt ind_neg_vel(ind_beg_neg(0)) then begin 
 for kk=0, nscans_neg-1 do begin
   debut(2*kk) = ind_neg_vel(ind_beg_neg(kk))
   fin(2*kk) = ind_neg_vel(ind_end_neg(kk))
 endfor
 ;  
 for kk=0, nscans_pos-1 do begin
   debut(2*kk+1) = ind_pos_vel(ind_beg_pos(kk))
   fin(2*kk+1) = ind_pos_vel(ind_end_pos(kk))
 endfor
endif


; On extrait les donnees qui nous interessent et on les sauve dans les fichiers adequat

print, "number of subscans", nscans

for k=0,nscans-1 do begin

;	reduce_donneestruct, datastr_in, fin(k)-debut(k)+1, datastr, /cube_orig
	reduce_donneestruct, datastr_in, fin(k)-debut(k)+1, datastr
	datastr.cube = datastr_in.cube(*,*,debut(k):fin(k))
	datastr.azim = datastr_in.azim(debut(k):fin(k))
	datastr.elev = datastr_in.elev(debut(k):fin(k))
	datastr.rota = datastr_in.rota(debut(k):fin(k))
	datastr.poschop = datastr_in.poschop(debut(k):fin(k))
	datastr.chopbeam = datastr_in.chopbeam(debut(k):fin(k))
	datastr.lst = datastr_in.lst(debut(k):fin(k))
	datastr.datapar = datastr_in.datapar(debut(k):fin(k))
;	datastr.cube_orig = datastr_in.cube_orig(*,*,debut(k):fin(k))
	
	if (n_elements(datastr.datapar.elevatio) gt 1) then begin
	  datastr.elevatio = median(datastr.datapar.elevatio)
	endif else begin
          datastr.elevatio = datastr.datapar.elevatio
	endelse
	
	datastr.subscan = strtrim(string(k+1), 2)
	datastr.nobs = fix(nscans)
	datastr.nsubs = fix(nscans)
	
	save, filename=dirdef + proj_name + '_' + strtrim(string(datastr.scannum),2) + '_' + datastr.subscan + '.xdr', datastr, /xdr
	print, 'SLICE_OTF : writing basic_xdr file for subscan # ' + datastr.subscan
	
endfor

; Pour test de validite
;latmean = fltarr(nscans) & latrms = fltarr(nscans)
;for k=0,nscans-1 do begin latmean(k) = mean(latoff(debut(k):fin(k))) & latrms(k) = sigma(latoff(debut(k):fin(k)))

return
end

